@@ -19,6 +19,7 @@ package compose
 import (
 	"fmt"
 	"io"
+	"os"
 	"strings"
 	"testing"
 	"time"
@@ -28,19 +29,20 @@ import (
 	"gotest.tools/v3/icmd"
 
 	"github.com/containerd/log"
+	"github.com/containerd/nerdctl/mod/tigron/expect"
+	"github.com/containerd/nerdctl/mod/tigron/test"
 
 	"github.com/containerd/nerdctl/v2/cmd/nerdctl/helpers"
 	"github.com/containerd/nerdctl/v2/pkg/composer/serviceparser"
 	"github.com/containerd/nerdctl/v2/pkg/rootlessutil"
 	"github.com/containerd/nerdctl/v2/pkg/testutil"
+	"github.com/containerd/nerdctl/v2/pkg/testutil/nerdtest"
 	"github.com/containerd/nerdctl/v2/pkg/testutil/nettestutil"
 )
 
 func TestComposeUp(t *testing.T) {
 	base := testutil.NewBase(t)
 	helpers.ComposeUp(t, base, fmt.Sprintf(`
-version: '3.1'
-
 services:
 
   wordpress:
@@ -101,7 +103,7 @@ COPY index.html /usr/share/nginx/html/index.html
 	base.ComposeCmd("-f", comp.YAMLFullPath(), "up", "-d", "--build").AssertOK()
 	defer base.ComposeCmd("-f", comp.YAMLFullPath(), "down", "-v").Run()
 
-	resp, err := nettestutil.HTTPGet("http://127.0.0.1:8080", 50, false)
+	resp, err := nettestutil.HTTPGet("http://127.0.0.1:8080", 5, false)
 	assert.NilError(t, err)
 	respBody, err := io.ReadAll(resp.Body)
 	assert.NilError(t, err)
@@ -116,8 +118,6 @@ func TestComposeUpNetWithStaticIP(t *testing.T) {
 	base := testutil.NewBase(t)
 	staticIP := "172.20.0.12"
 	var dockerComposeYAML = fmt.Sprintf(`
-version: '3.1'
-
 services:
   svc0:
     image: %s
@@ -154,8 +154,6 @@ func TestComposeUpMultiNet(t *testing.T) {
 	base := testutil.NewBase(t)
 
 	var dockerComposeYAML = fmt.Sprintf(`
-version: '3.1'
-
 services:
   svc0:
     image: %s
@@ -203,8 +201,6 @@ func TestComposeUpOsEnvVar(t *testing.T) {
 	base := testutil.NewBase(t)
 	const containerName = "nginxAlpine"
 	var dockerComposeYAML = fmt.Sprintf(`
-version: '3.1'
-
 services:
   svc1:
     image: %s
@@ -236,8 +232,6 @@ func TestComposeUpDotEnvFile(t *testing.T) {
 	base := testutil.NewBase(t)
 
 	var dockerComposeYAML = `
-version: '3.1'
-
 services:
   svc3:
     image: ghcr.io/stargz-containers/nginx:$TAG
@@ -259,8 +253,6 @@ func TestComposeUpEnvFileNotFoundError(t *testing.T) {
 	base := testutil.NewBase(t)
 
 	var dockerComposeYAML = `
-version: '3.1'
-
 services:
   svc4:
     image: ghcr.io/stargz-containers/nginx:$TAG
@@ -283,13 +275,11 @@ func TestComposeUpWithScale(t *testing.T) {
 	base := testutil.NewBase(t)
 
 	var dockerComposeYAML = fmt.Sprintf(`
-version: '3.1'
-
 services:
   test:
     image: %s
     command: "sleep infinity"
-`, testutil.AlpineImage)
+`, testutil.CommonImage)
 
 	comp := testutil.NewComposeDir(t, dockerComposeYAML)
 	defer comp.CleanUp()
@@ -306,8 +296,6 @@ func TestComposeIPAMConfig(t *testing.T) {
 	base := testutil.NewBase(t)
 
 	var dockerComposeYAML = fmt.Sprintf(`
-version: '3.1'
-
 services:
   foo:
     image: %s
@@ -318,7 +306,7 @@ networks:
     ipam:
       config:
         - subnet: 10.1.100.0/24
-`, testutil.AlpineImage)
+`, testutil.CommonImage)
 
 	comp := testutil.NewComposeDir(t, dockerComposeYAML)
 	defer comp.CleanUp()
@@ -336,20 +324,18 @@ func TestComposeUpRemoveOrphans(t *testing.T) {
 
 	var (
 		dockerComposeYAMLOrphan = fmt.Sprintf(`
-version: '3.1'
-
 services:
   test:
     image: %s
     command: "sleep infinity"
-`, testutil.AlpineImage)
+`, testutil.CommonImage)
 
 		dockerComposeYAMLFull = fmt.Sprintf(`
 %s
   orphan:
     image: %s
     command: "sleep infinity"
-`, dockerComposeYAMLOrphan, testutil.AlpineImage)
+`, dockerComposeYAMLOrphan, testutil.CommonImage)
 	)
 
 	compOrphan := testutil.NewComposeDir(t, dockerComposeYAMLOrphan)
@@ -374,13 +360,11 @@ func TestComposeUpIdempotent(t *testing.T) {
 	base := testutil.NewBase(t)
 
 	var dockerComposeYAML = fmt.Sprintf(`
-version: '3.1'
-
 services:
   test:
     image: %s
     command: "sleep infinity"
-`, testutil.AlpineImage)
+`, testutil.CommonImage)
 
 	comp := testutil.NewComposeDir(t, dockerComposeYAML)
 	defer comp.CleanUp()
@@ -393,12 +377,51 @@ services:
 	base.ComposeCmd("-f", comp.YAMLFullPath(), "down").AssertOK()
 }
 
+func TestComposeUpNoRecreateDependencies(t *testing.T) {
+	base := testutil.NewBase(t)
+
+	var dockerComposeYAML = fmt.Sprintf(`
+services:
+  foo:
+    image: %s
+    command: "sleep infinity"
+  bar:
+    image: %s
+    command: "sleep infinity"
+    depends_on:
+      - foo
+`, testutil.CommonImage, testutil.CommonImage)
+
+	comp := testutil.NewComposeDir(t, dockerComposeYAML)
+	defer comp.CleanUp()
+	projectName := comp.ProjectName()
+	t.Logf("projectName=%q", projectName)
+
+	base.ComposeCmd("-f", comp.YAMLFullPath(), "up", "-d", "foo").AssertOK()
+	defer base.ComposeCmd("-f", comp.YAMLFullPath(), "down", "-v").Run()
+
+	fooName := serviceparser.DefaultContainerName(projectName, "foo", "1")
+	id1Cmd := base.Cmd("inspect", fooName, "--format", "{{.Id}}")
+	id1Res := id1Cmd.Run()
+	out1 := strings.TrimSpace(id1Res.Stdout())
+	assert.Assert(id1Cmd.Base.T, id1Res.ExitCode == 0, id1Res.Stdout()+id1Res.Stderr())
+
+	// Bring up dependent service; ensure foo is not recreated (ID unchanged)
+	base.ComposeCmd("-f", comp.YAMLFullPath(), "up", "-d", "bar").AssertOK()
+
+	id2Cmd := base.Cmd("inspect", fooName, "--format", "{{.Id}}")
+	id2Res := id2Cmd.Run()
+	out2 := strings.TrimSpace(id2Res.Stdout())
+	assert.Assert(id2Cmd.Base.T, id2Res.ExitCode == 0, id2Res.Stdout()+id2Res.Stderr())
+
+	assert.Equal(base.T, out1, out2)
+}
+
 func TestComposeUpWithExternalNetwork(t *testing.T) {
-	containerName1 := testutil.Identifier(t) + "-1"
-	containerName2 := testutil.Identifier(t) + "-2"
-	networkName := testutil.Identifier(t) + "-network"
-	var dockerComposeYaml1 = fmt.Sprintf(`
-version: "3"
+	testCase := nerdtest.Setup()
+
+	testCase.Setup = func(data test.Data, helpers test.Helpers) {
+		var dockerComposeYaml1 = fmt.Sprintf(`
 services:
   %s:
     image: %s
@@ -410,9 +433,8 @@ services:
 networks:
   %s:
     external: true
-`, containerName1, testutil.NginxAlpineImage, containerName1, networkName, networkName)
-	var dockerComposeYaml2 = fmt.Sprintf(`
-version: "3"
+`, data.Identifier("con-1"), testutil.NginxAlpineImage, data.Identifier("con-1"), data.Identifier("network"), data.Identifier("network"))
+		var dockerComposeYaml2 = fmt.Sprintf(`
 services:
   %s:
     image: %s
@@ -424,26 +446,34 @@ services:
 networks:
   %s:
     external: true
-`, containerName2, testutil.NginxAlpineImage, containerName2, networkName, networkName)
-	comp1 := testutil.NewComposeDir(t, dockerComposeYaml1)
-	defer comp1.CleanUp()
-	comp2 := testutil.NewComposeDir(t, dockerComposeYaml2)
-	defer comp2.CleanUp()
-	base := testutil.NewBase(t)
-	// Create the test network
-	base.Cmd("network", "create", networkName).AssertOK()
-	defer base.Cmd("network", "rm", networkName).Run()
-	// Run the first compose
-	base.ComposeCmd("-f", comp1.YAMLFullPath(), "up", "-d").AssertOK()
-	defer base.ComposeCmd("-f", comp1.YAMLFullPath(), "down", "-v").Run()
-	// Run the second compose
-	base.ComposeCmd("-f", comp2.YAMLFullPath(), "up", "-d").AssertOK()
-	defer base.ComposeCmd("-f", comp2.YAMLFullPath(), "down", "-v").Run()
-	// Down the second compose
-	base.ComposeCmd("-f", comp2.YAMLFullPath(), "down", "-v").AssertOK()
-	// Run the second compose again
-	base.ComposeCmd("-f", comp2.YAMLFullPath(), "up", "-d").AssertOK()
-	base.Cmd("exec", containerName1, "wget", "-qO-", "http://"+containerName2).AssertOutContains(testutil.NginxAlpineIndexHTMLSnippet)
+`, data.Identifier("con-2"), testutil.NginxAlpineImage, data.Identifier("con-2"), data.Identifier("network"), data.Identifier("network"))
+		tmp := data.Temp()
+
+		tmp.Save(dockerComposeYaml1, "project-1", "compose.yaml")
+		tmp.Save(dockerComposeYaml2, "project-2", "compose.yaml")
+
+		helpers.Ensure("network", "create", data.Identifier("network"))
+		helpers.Ensure("compose", "-f", tmp.Path("project-1", "compose.yaml"), "up", "-d")
+		helpers.Ensure("compose", "-f", tmp.Path("project-2", "compose.yaml"), "up", "-d")
+		helpers.Ensure("compose", "-f", tmp.Path("project-2", "compose.yaml"), "down", "-v")
+		helpers.Ensure("compose", "-f", tmp.Path("project-2", "compose.yaml"), "up", "-d")
+		nerdtest.EnsureContainerStarted(helpers, data.Identifier("con-2"))
+	}
+
+	testCase.Command = func(data test.Data, helpers test.Helpers) test.TestableCommand {
+		helpers.Ensure("exec", data.Identifier("con-1"), "cat", "/etc/hosts")
+		return helpers.Command("exec", data.Identifier("con-1"), "wget", "-qO-", "http://"+data.Identifier("con-2"))
+	}
+
+	testCase.Expected = test.Expects(0, nil, expect.Contains(testutil.NginxAlpineIndexHTMLSnippet))
+
+	testCase.Cleanup = func(data test.Data, helpers test.Helpers) {
+		helpers.Anyhow("compose", "-f", data.Temp().Path("project-1", "compose.yaml"), "down", "-v")
+		helpers.Anyhow("compose", "-f", data.Temp().Path("project-2", "compose.yaml"), "down", "-v")
+		helpers.Anyhow("network", "rm", data.Identifier("network"))
+	}
+
+	testCase.Run(t)
 }
 
 func TestComposeUpWithBypass4netns(t *testing.T) {
@@ -456,8 +486,6 @@ func TestComposeUpWithBypass4netns(t *testing.T) {
 	testutil.RequireSystemService(t, "bypass4netnsd")
 	base := testutil.NewBase(t)
 	helpers.ComposeUp(t, base, fmt.Sprintf(`
-version: '3.1'
-
 services:
 
   wordpress:
@@ -499,6 +527,12 @@ func TestComposeUpProfile(t *testing.T) {
 	serviceRegular := testutil.Identifier(t) + "-regular"
 	serviceProfiled := testutil.Identifier(t) + "-profiled"
 
+	// write the env.common file to tmpdir
+	tmpDir := t.TempDir()
+	envFilePath := fmt.Sprintf("%s/env.common", tmpDir)
+	err := os.WriteFile(envFilePath, []byte("TEST_ENV_INJECTION=WORKS\n"), 0644)
+	assert.NilError(t, err)
+
 	dockerComposeYAML := fmt.Sprintf(`
 services:
   %s:
@@ -508,7 +542,9 @@ services:
     image: %[3]s
     profiles:
       - test-profile
-`, serviceRegular, serviceProfiled, testutil.NginxAlpineImage)
+    env_file:
+      - %[4]s
+`, serviceRegular, serviceProfiled, testutil.NginxAlpineImage, envFilePath)
 
 	// * Test with profile
 	//   Should run both the services:
@@ -521,6 +557,10 @@ services:
 	psCmd := base.Cmd("ps", "-a", "--format={{.Names}}")
 	psCmd.AssertOutContains(serviceRegular)
 	psCmd.AssertOutContains(serviceProfiled)
+
+	execCmd := base.ComposeCmd("-f", comp1.YAMLFullPath(), "exec", serviceProfiled, "env")
+	execCmd.AssertOutContains("TEST_ENV_INJECTION=WORKS")
+
 	base.ComposeCmd("-f", comp1.YAMLFullPath(), "--profile", "test-profile", "down", "-v").AssertOK()
 
 	// * Test without profile
@@ -544,8 +584,6 @@ func TestComposeUpAbortOnContainerExit(t *testing.T) {
 services:
   %s:
     image: %s
-    ports:
-      - 8080:80
   %s:
     image: %s
     entrypoint: /bin/sh -c "exit 1"
